#Vertical functions run in each column of MONARCHS model
#Includes heat equation solver ('heateqn'), surface boundary counditions ('sfc_flux', 'sfc_albedo', 'bulk_fluxes') and vertical percolation of water ('')
import numpy as np

def heateqn(x, cell, dt, dz, LW_in, SW_in, T_air, p_air, s_hum, wind):
    # output=[]
    output = np.zeros(len(x))

    k_ice = (1000 * 2.24 * 0.001 + 5.975 * 0.000001 * (273.15 - cell.T) ** 1.156)
    k = cell.Sfrac * k_ice + (1 - cell.Sfrac - cell.Lfrac) * cell.k_air + cell.Lfrac * cell.k_water;
    cp_ice = [(1000 * 0.00716 * i + 0.138) for i in cell.T]
    cp = cell.Sfrac * cp_ice + (1 - cell.Sfrac - cell.Lfrac) * cell.cp_air + cell.Lfrac * cell.cp_water;
    beta = k / (cp)
    epsilon = 0.98
    sigma = 5.670373 * (10 ** -8)

    output[0] = k[0] * (x[0] - x[1]) / dz - (sfc_flux(LW_in, SW_in, T_air, p_air, s_hum, wind, cell, x) -
                                             epsilon * sigma * x[0] ** 4)

    idx = np.arange(1, len(x) - 1)

    output[idx] = cell.T[idx] - x[idx] + dt * (beta[idx] / dz ** 2) * (x[idx + 1] - 2 * x[idx] + x[idx - 1])
    output[-1] = cell.T[len(x) - 1] - x[len(x) - 1] + dt * (beta[len(x) - 1] / dz ** 2) * (
                -x[len(x) - 1] + x[len(x) - 2])

    return output


def sfc_flux(LW_in, SW_in, T_air, p_air, s_hum, wind, cell, x):
    # Positive going into ice shelf
    alpha = sfc_albedo(cell)
    Flat, Fsens = bulk_fluxes(cell, wind, T_air, x[0], s_hum, p_air)
    epsilon = 0.98
    Q = epsilon * LW_in + (1 - alpha) * SW_in + Flat + Fsens
    # print('Q= ',Q, ' albedo=', alpha, ' F_sens=', Fsens, ' Flat=', Flat, 'F_lw=', LW_in, 'Fsw=', SW_in)
    return Q


def sfc_albedo(cell):
    # Calculate surface albedo
    if cell.melt == 'True':
        if cell.exposed_water == 'True':
            if cell.lid == 'True':
                alpha = 0.413  # ice lid albedo
            elif cell.lake == 'True':
                h = cell.lake_depth;
                alpha = (9702 + 1000 * exp(3.6 * h)) / (-539 + 20000 * exp(3.6 * h));  # lake albedo
        else:
            alpha = 0.6  # wet snow albedo
    else:
        alpha = 0.8670  # dry snow albedo

    return alpha


def bulk_fluxes(cell, wind, T_air, T_sfc, s_hum, p_air):
    g = 9.8;  # Gravity
    b = 20;
    dz = 10;  # Height windspeed is measured at
    CT0 = 1.3 * 10 ** (-3);
    c = 1961 * b * CT0;
    if wind == 0:  # Richardson number
        Ri = 0;
    else:
        Ri = (g * (T_air - T_sfc) * dz) / (T_air * wind ** 2);
    if Ri < 0:
        CT = CT0 * (1 - (2 * b * Ri) / (1 + c * abs(Ri ** 0.5)));
    else:
        CT = CT0 * (1 + b * Ri) ** (-2);
    L = 2.501 * 10 ** 6;
    p_v = 2.53 * 10 ** 8 * np.exp(-5420 / T_sfc);
    q_0 = (0.622 * p_v) / (p_air - 0.378 * p_v);
    Fsens = 1.275 * 1005 * CT * wind * (T_air - T_sfc);
    Flat = 1.275 * L * CT * wind * ((s_hum / 1000) - q_0);
    # print('ri=',Ri,'CT=',CT,'p_v=',p_v,'s_hum=',s_hum)
    return Flat, Fsens





